/* eckcdsa/eckcdsa.h */
/*
 * EC-KCDSA source code by Yeon-Hyeong Yang <yhyang@oberon.postech.ac.kr>
 */

#ifndef ECKCDSA_H
#define ECKCDSA_H

#undef OPENSSL_NO_ECKCDSA

#include <openssl/ec.h>

#ifndef OPENSSL_NO_DEPRECATED
#include <openssl/bn.h>
#endif

#ifdef  __cplusplus
extern "C" {
#endif

typedef struct eckcdsa_st ECKCDSA;
typedef struct eckcdsa_method_st ECKCDSA_METHOD;
typedef struct eckcdsa_sig_st ECKCDSA_SIG;

/* primary interface functions */
ECKCDSA *ECKCDSA_new(void);
void ECKCDSA_free(ECKCDSA *eckcdsa);
ECKCDSA_SIG * ECKCDSA_SIG_new(void);
void ECKCDSA_SIG_free(ECKCDSA_SIG *sig);
ECKCDSA_SIG * ECKCDSA_do_sign(const unsigned char *mesg, size_t mlen,
		ECKCDSA *eckcdsa);
int ECKCDSA_sign_setup(ECKCDSA *eckcdsa, BN_CTX *ctx_in, BIGNUM **kp,
		unsigned char **rp, size_t *rlenp);
int ECKCDSA_do_verify(const unsigned char *mesg, size_t mlen,
		ECKCDSA_SIG *sig, ECKCDSA *eckcdsa);

/* interface functions related to key and parameters */
int ECKCDSA_generate_parameters(ECKCDSA *eckcdsa, const EVP_MD * evp_md, int nid);
int ECKCDSA_generate_key(ECKCDSA *eckcdsa);

/* accessing method structure */
const ECKCDSA_METHOD * ECKCDSA_get_default_method();

/* ISO method */
const ECKCDSA_METHOD * ECKCDSA_iso_method();

/* minimal UI functions */
void ECKCDSA_print(BIO *out, const ECKCDSA *eckcdsa, int off);

#ifdef  __cplusplus
}
#endif
#endif /* ECKCDSA_H */
