/* eckcdsa/eckcdsa_lcl.h */
/* 
 * EC-KCDSA source code by Yeon-Hyeong Yang <yhyang@oberon.postech.ac.kr>
 */

#ifndef ECKCDSA_LCL_H
#define ECKCDSA_LCL_H

#include "eckcdsa.h"

struct eckcdsa_st
{
	/* EC_KEY contains all relevant domain parameters except the hash function
	 * identifier. This is rather strange. OpenSSL defines EC_KEY structure in
	 * ec module, not in ecdsa. However, all related functions are stuck to
	 * ECDSA and internal structure of EC_KEY is not available to the outside
	 * functions. so, it is inevitable to mimick some essential features and
	 * implement here again. */

	/* domain parameters */
	const EVP_MD *evp_md; /* domain specific hash algorithm */
	EC_GROUP *group; /* all necessary curve informations is here */

	/* key pair */
	BIGNUM *priv_key;
	EC_POINT *pub_key;

	point_conversion_form_t conv_form;

	/* precomputed signature parts */
	BIGNUM *k; /* precomputed randomizer */
	unsigned char *r; /* precomputed witness */
	size_t rlen; /* lentgh of r */

	/* internal method functions */
	const ECKCDSA_METHOD *meth;
};

struct eckcdsa_method_st
{
	const char *name;
	ECKCDSA_SIG * (*eckcdsa_do_sign)(const unsigned char *mesg, int mlen,
			ECKCDSA *eckcdsa);
	int (*eckcdsa_sign_setup)(ECKCDSA *eckcdsa, BN_CTX *ctx_in, BIGNUM **kp,
			unsigned char **rp, size_t *rlenp);
	int (*eckcdsa_do_verify)(const unsigned char *mesg, int mlen,
			ECKCDSA_SIG *sig, ECKCDSA *eckcdsa);
	int (*eckcdsa_gen_param)(ECKCDSA *eckcdsa, const EVP_MD * evp_md, int nid);
	int (*eckcdsa_gen_key)(ECKCDSA *eckcdsa);
};

/* same as KCDSA_SIG */
struct eckcdsa_sig_st
{
	unsigned char *r;
	size_t rlen;
	BIGNUM *s;
};

/* ISO method */
ECKCDSA_SIG * eckcdsa_do_sign(const unsigned char *mesg, int mlen,
		ECKCDSA *eckcdsa);
int eckcdsa_sign_setup(ECKCDSA *eckcdsa, BN_CTX *ctx_in, BIGNUM **kp,
		unsigned char **rp, size_t *rlenp);
int eckcdsa_do_verify(const unsigned char *mesg, int mlen,
		ECKCDSA_SIG *sig, ECKCDSA *eckcdsa);
int eckcdsa_gen_param(ECKCDSA *eckcdsa, const EVP_MD * evp_md, int nid);
int eckcdsa_gen_key(ECKCDSA *eckcdsa);

#endif /* ECKCDSA_LCL_H */
