/* kcdsa/kcdsa_iso.c */

/*
 * KCDSA source code by Yeon-Hyeong Yang <yhyang@oberon.postech.ac.kr>
 */

#include <openssl/evp.h>

#include "kcdsa_lcl.h"

static KCDSA_METHOD iso_kcdsa_meth = {
	"ISO KCDSA method",
	kcdsa_do_sign,
	kcdsa_sign_setup,
	kcdsa_do_verify,
	NULL, /* kcdsa_mod_exp, */
	NULL, /* kcdsa_bn_mod_exp, */
	kcdsa_init,
	kcdsa_finish,
	0,
	NULL,
	kcdsa_iso_paramgen,
	kcdsa_iso_keygen
};

const KCDSA_METHOD *KCDSA_iso_method(void)
{
	return &iso_kcdsa_meth;
}

int kcdsa_init(KCDSA *kcdsa)
{
	kcdsa->evp_md = EVP_sha1();
	kcdsa->flags|=KCDSA_FLAG_CACHE_MONT_P;
	return(1);
}

int kcdsa_finish(KCDSA *kcdsa)
{
	if (kcdsa->evp_md != NULL) kcdsa->evp_md = NULL;
	if(kcdsa->method_mont_p)
		BN_MONT_CTX_free(kcdsa->method_mont_p);
	return(1);
}

