/* kcdsa/kcdsa_lcl.h */
/*
 * KCDSA source code by Yeon-Hyeong Yang <yhyang@oberon.postech.ac.kr>
 */

#ifndef KCDSA_LCL_H
#define KCDSA_LCL_H

#include "kcdsa.h"

/* dsa_method로부터 수정함. */
struct kcdsa_method
	{
	const char *name;
	KCDSA_SIG * (*kcdsa_do_sign)(const unsigned char *mesg, int mlen,
			KCDSA *kcdsa);
	int (*kcdsa_sign_setup)(KCDSA *kcdsa, BN_CTX *ctx_in,
			BIGNUM **kp, unsigned char **rp, size_t *rlenp);
	int (*kcdsa_do_verify)(const unsigned char *mesg, int mlen,
			KCDSA_SIG *sig, KCDSA *kcdsa);
	int (*kcdsa_mod_exp)(KCDSA *kcdsa, BIGNUM *rr, BIGNUM *a1, BIGNUM *p1,
			BIGNUM *a2, BIGNUM *p2, BIGNUM *m, BN_CTX *ctx,
			BN_MONT_CTX *in_mont);
	int (*bn_mod_exp)(KCDSA *kcdsa, BIGNUM *r, BIGNUM *a, const BIGNUM *p,
				const BIGNUM *m, BN_CTX *ctx,
				BN_MONT_CTX *m_ctx); /* Can be null */

	int (*init)(KCDSA *kcdsa);
	int (*finish)(KCDSA *kcdsa);
	int flags;
	char *app_data;
	/* If this is non-NULL, it is used to generate KCDSA parameters */
	int (*kcdsa_paramgen)(KCDSA *kcdsa, int pbits, int qbits,
			unsigned char *seed, int seed_len, const EVP_MD *evp_md,
			int *counter_ret, unsigned long *h_ret,
			BN_GENCB *cb);
	/* If this is non-NULL, it is used to generate KCDSA keys */
	int (*kcdsa_keygen)(KCDSA *kcdsa);
	};

struct kcdsa_st
	{
	/* This first variable is used to pick up errors where
	 * a KCDSA is passed instead of of a EVP_PKEY */
	int pad;
	long version;
	int write_params;
	BIGNUM *p;
	BIGNUM *q;
	BIGNUM *g;
	const EVP_MD *evp_md; /* domain specific hash algorithm */

	BIGNUM *pub_key;  /* y public key */
	BIGNUM *priv_key; /* x private key */

	BIGNUM *k;	/* Signing pre-calc */
	unsigned char *r;	/* Signing pre-calc */
	size_t rlen;

	int flags;
	/* Normally used to cache montgomery values */
	BN_MONT_CTX *method_mont_p;
	//int references;
	//CRYPTO_EX_DATA ex_data;
	const KCDSA_METHOD *meth;
	/* functional reference if 'meth' is ENGINE-provided */
	//ENGINE *engine;
	};

/* ISO method */
KCDSA_SIG *kcdsa_do_sign(const unsigned char *mesg, int mlen, KCDSA *kcdsa);
int kcdsa_sign_setup(KCDSA *kcdsa, BN_CTX *ctx_in,
		BIGNUM **kp, unsigned char **rp, size_t *rlenp);
int kcdsa_do_verify(const unsigned char *mesg, int mlen,
		KCDSA_SIG *sig, KCDSA *kcdsa);
int kcdsa_init(KCDSA *kcdsa);
int kcdsa_finish(KCDSA *kcdsa);
int kcdsa_iso_paramgen(KCDSA *ret, int pbits, int qbits,
		unsigned char *seed_in, int seed_len, const EVP_MD *evp_md,
		int *counter_ret, unsigned long *h_ret, BN_GENCB *cb);
int kcdsa_iso_keygen(KCDSA *kcdsa);

/* TTA method */
int kcdsa_tta_paramgen(KCDSA *ret, int pbits, int qbits,
		unsigned char *seed_in, int seed_len, const EVP_MD *evp_md,
		int *counter_ret, unsigned long *h_ret, BN_GENCB *cb);
int kcdsa_tta_keygen(KCDSA *kcdsa);

int kcdsa_tta_prng(unsigned char *seed, size_t seed_len, const EVP_MD *evp_md,
		unsigned char *out, size_t out_len);

#endif
