/* kcdsa/kcdsa_tta.c */

/*
 * KCDSA source code by Yeon-Hyeong Yang <yhyang@oberon.postech.ac.kr>
 */

#include <string.h>
#include <openssl/evp.h>

#include "kcdsa_lcl.h"

static KCDSA_METHOD tta_kcdsa_meth = {
	"TTA KCDSA method",
	kcdsa_do_sign, /* ISO method */
	kcdsa_sign_setup, /* ISO method */
	kcdsa_do_verify, /* ISO method */
	NULL, /* kcdsa_mod_exp, */
	NULL, /* kcdsa_bn_mod_exp, */
	kcdsa_init, /* ISO method */
	kcdsa_finish, /* ISO method */
	0,
	NULL,
	kcdsa_tta_paramgen,
	NULL /* kcdsa_tta_keygen */
};

const KCDSA_METHOD *KCDSA_tta_method(void)
{
	return &tta_kcdsa_meth;
}

int kcdsa_tta_prng(unsigned char *seed, size_t seed_len, const EVP_MD *evp_md,
		unsigned char *out, size_t out_len)
{
	int i,k,ret=0;
	int md_len = EVP_MD_size(evp_md);
	unsigned char buf[KCDSA_MAX_Q_LENGTH+1];
	unsigned char md[KCDSA_MAX_Q_LENGTH];
	BIGNUM tmp, W;

	if (md_len > KCDSA_MAX_Q_LENGTH) return 0;

	BN_init(&tmp);
	BN_init(&W);

	memcpy(buf, seed, seed_len);
	k = out_len/md_len;
	for (i = 0; i <= k; i++) {
		buf[seed_len] = i; /* seed||i */
		EVP_Digest(buf, seed_len+1, md, NULL, evp_md, NULL);
		if (BN_bin2bn(md, md_len, &tmp) == NULL) goto prng_err;
		if (!BN_lshift(&tmp,&tmp,md_len*i)) goto prng_err;
		if (!BN_add(&W, &W, &tmp)) goto prng_err;
	}
	if (!BN_mask_bits(&W, out_len)) goto prng_err;
	if (!BN_bn2bin(&W, out)) goto prng_err;

	ret = 1;

prng_err:
	BN_clear_free(&tmp);
	BN_clear_free(&W);

	return ret;
}

