/* kcdsa/kcdsatest.c */

/* Until the key-gen callbacks are modified to use newer prototypes, we allow
 * deprecated functions for openssl-internal code */
#ifdef OPENSSL_NO_DEPRECATED
#undef OPENSSL_NO_DEPRECATED
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <openssl/crypto.h>
#include <openssl/rand.h>
#include <openssl/bio.h>
#include <openssl/err.h>
#include <openssl/bn.h>

#include <openssl/evp.h>

/* openssl 방식을 따르면, kcdsa.h에 있는 struct의 member variable들은 외부에서
 * 볼 수 있으나 kcdsa_lcl.h에 있는 것은 외부에서 볼 수 없다. 코드는 시험용이기
 * 때문에 kcdsa.h를 include하지 않고 kcdsa_lcl.h를 include한다. */
#define TEST

#ifdef TEST
#include "kcdsa_lcl.h"
#else
#include "kcdsa.h"
#endif

/* 파라미터 생성 과정을 화면에 출력하기 위해 필요한 함수이다. 실제의 동작에
 * 영향을 주는 요소는 아니다. */
static int kcdsa_cb(int p, int n, BN_GENCB *arg);

/* TODO: hash function은 SHA-1을 쓰기로 함 */

/* seed로부터 만들어지는 domain parameter들을 검증해 보기 위한 데이터들이다.
 * 그러나 KCDSA의 경우 비교해 볼 국내 표준의 참고구현값들은 HAS160을 사용한
 * 것들이고, 국제 표준인 14888-3의 경우는 파라미터 생성과정은 빠져 있다. */

#if 0
/* seed, out_p, out_q, out_g are taken from the updated Appendix 5 to
 * FIPS PUB 186 and also appear in Appendix 5 to FIPS PIB 186-1 */
static unsigned char seed[20]={
	0xd5,0x01,0x4e,0x4b,0x60,0xef,0x2b,0xa8,0xb6,0x21,0x1b,0x40,
	0x62,0xba,0x32,0x24,0xe0,0x42,0x7d,0xd3,
	};
#endif

/* 160-bit seed by Lbird */
static unsigned char seed[20]={
	0x06,0xdf,0x94,0xfc,0x21,0x53,0xad,0xb7,
	0xe6,0xa4,0x3a,0x92,0x59,0x3c,0xdd,0x4d,
	0x2b,0x6a,0xc0,0x95,
	};

/* 1024-bit p from the seed */
static unsigned char out_p[]={
	0xea,0x95,0x37,0x87,0x4f,0xe0,0x08,0x39,
	0x47,0x3a,0x99,0x29,0x77,0xc0,0x9d,0x55,
	0x24,0x7e,0x70,0x48,0x53,0x4a,0x83,0x93,
	0xc0,0xf8,0xbc,0x84,0xa6,0x1c,0x72,0xca,
	0x98,0xda,0x8f,0x0c,0x99,0x77,0x0a,0x03,
	0x9e,0x3a,0x4c,0xee,0x4a,0x1d,0x56,0x6b,
	0xf7,0x88,0x04,0xda,0xa5,0xda,0x08,0x9e,
	0x92,0xeb,0x59,0x55,0xd0,0xc7,0x18,0x03,
	0x7e,0x3c,0xa8,0x7a,0x26,0x2c,0x99,0x81,
	0xfd,0x34,0x17,0x15,0x3a,0x4d,0x99,0xf4,
	0xab,0x9c,0xed,0x5d,0x47,0xc2,0xc0,0x0d,
	0x77,0x3b,0x5f,0xca,0xb6,0x00,0xa4,0xe8,
	0xf2,0x8e,0x2b,0x2d,0x4b,0x09,0xe1,0x13,
	0xc2,0x17,0x59,0xbd,0xc3,0x35,0xbd,0x1f,
	0x28,0x91,0x06,0x12,0x20,0x86,0x9c,0x47,
	0x9b,0x66,0xa1,0x8a,0x1e,0x40,0x0e,0xd3,
	};

/* 160-bit q from the seed */
static unsigned char out_q[]={
	0xc6,0x35,0xab,0x0f,0xc0,0x95,0xfd,0x4c,
	0xe1,0x67,0x88,0x4e,0x09,0x9f,0xfe,0x3e,
	0x42,0x57,0x39,0xfd,
	};

/* 1024-bit g from the seed */
static unsigned char out_g[]={
	0xc8,0xf5,0xe2,0xa1,0x0f,0xfe,0x78,0x93,
	0xd0,0xd0,0xee,0x3c,0x05,0x45,0x8a,0x98,
	0xc1,0x3d,0xd2,0xf2,0x90,0x8b,0xa3,0xb9,
	0x4f,0xf5,0x8d,0xe0,0x07,0x98,0x5e,0xab,
	0x8d,0x81,0x20,0x15,0xf2,0x7b,0x69,0x5d,
	0xe9,0xa6,0xca,0xc1,0x05,0x1b,0x78,0x35,
	0xe0,0xdb,0x04,0x2e,0x91,0x52,0xb0,0xe0,
	0x13,0x89,0x09,0xf1,0xa1,0xb0,0xed,0xce,
	0xcf,0xcc,0x0a,0x94,0x81,0x7b,0x00,0x4e,
	0x76,0xf0,0xdd,0x62,0xb7,0x54,0xef,0xc6,
	0xce,0x09,0x23,0xe2,0x84,0x94,0x0c,0xef,
	0x61,0x2f,0x12,0x4e,0x76,0x23,0x45,0xe4,
	0xac,0xbd,0xb4,0x5e,0x91,0x84,0x34,0xfa,
	0x6a,0x3a,0x2b,0x0a,0x96,0xec,0x88,0x10,
	0x86,0xab,0x1b,0x89,0x84,0xd2,0xc3,0xe6,
	0x2d,0x4b,0x3c,0x90,0x42,0xb9,0x63,0x6c,
	};

static const unsigned char str1[]="12345678901234567890";

static const char rnd_seed[] = "string to make the random number generator think it has entropy";

/* stderr를 대신함 */
static BIO *bio_err=NULL;

int main(int argc, char **argv)
	{
	BN_GENCB cb;
	KCDSA *kcdsa=NULL;
	KCDSA_SIG *kcdsa_sig = NULL;
	int counter,ret=0,i,j;
	unsigned char buf[256];
	unsigned long h;
	//unsigned char sig[256];
	//unsigned int siglen;

	if (bio_err == NULL)
		bio_err=BIO_new_fp(stderr,BIO_NOCLOSE);

	CRYPTO_malloc_debug_init();
	CRYPTO_dbg_set_options(V_CRYPTO_MDEBUG_ALL);
	CRYPTO_mem_ctrl(CRYPTO_MEM_CHECK_ON);

	ERR_load_crypto_strings();
	RAND_seed(rnd_seed, sizeof rnd_seed);

	/* openssl의 string 입출력은 BIO module을 통해서 이루어짐 */
	BIO_printf(bio_err,"test generation of KCDSA parameters\n");

	BN_GENCB_set(&cb, kcdsa_cb, bio_err);
	/* KCDSA는 두가지 방식이 있는데 ISO method와 TTA method이다. iso method와
	 * tta method는 parameter generation에서만 차이가 난다. 서명 생성,
	 * 검증에서는 동일하다. 기본적으로는 iso method를 사용한다. */
	if(((kcdsa = KCDSA_new()) == NULL) ||
			!KCDSA_generate_parameters_ex(kcdsa, 1024, 160, seed, 20, EVP_sha1(), &counter, &h, &cb))
		goto end;

	BIO_printf(bio_err,"seed\n");
	for (i=0; i<20; i+=4)
		{
		BIO_printf(bio_err,"%02X%02X%02X%02X ",
			seed[i],seed[i+1],seed[i+2],seed[i+3]);
		}
	BIO_printf(bio_err,"\ncounter=%d h=%ld\n",counter,h);
		
	if (kcdsa == NULL) goto end;
#ifdef TEST
	if (counter != 16) 
		{
		BIO_printf(bio_err,"counter should be 16\n");
		goto end;
		}
	if (h != 2)
		{
		BIO_printf(bio_err,"h should be 2\n");
		goto end;
		}
#endif

#ifdef TEST
	i=BN_bn2bin(kcdsa->q,buf);
	j=sizeof(out_q);
	if ((i != j) || (memcmp(buf,out_q,i) != 0))
		{
		BIO_printf(bio_err,"q value is wrong\n");
		goto end;
		}
#endif

	i=BN_bn2bin(kcdsa->p,buf);
	j=sizeof(out_p);
	if ((i != j) || (memcmp(buf,out_p,i) != 0))
		{
		BIO_printf(bio_err,"p value is wrong\n");
		goto end;
		}

	i=BN_bn2bin(kcdsa->g,buf);
	j=sizeof(out_g);
	if ((i != j) || (memcmp(buf,out_g,i) != 0))
		{
		BIO_printf(bio_err,"g value is wrong\n");
		goto end;
		}

	kcdsa->flags |= KCDSA_FLAG_NO_EXP_CONSTTIME;
	KCDSA_generate_key(kcdsa);
	KCDSA_print(bio_err,kcdsa,0);
	kcdsa_sig = KCDSA_do_sign(str1, 20, kcdsa);
	if (KCDSA_do_verify(str1, 20, kcdsa_sig, kcdsa) == 1)
		ret=1;

	/* clear kcdsa_sig and restart */
	if (kcdsa_sig != NULL) KCDSA_SIG_free(kcdsa_sig);
	kcdsa_sig = NULL;

	/*
	kcdsa->flags &= ~KCDSA_FLAG_NO_EXP_CONSTTIME;
	KCDSA_generate_key(kcdsa);
	KCDSA_print(bio_err,kcdsa,0);
	kcdsa_sig = KCDSA_do_sign(str1, 20, kcdsa);
	if (KCDSA_verify(0, str1, 20, sig, siglen, kcdsa) == 1)
		ret=1;
		*/

end:
	/* 원래 ERR_print_errors()에 의해서 모든 에러 메시지가 출력되어야 하지만,
	 * kcdsa 관련한 에러 메시지는 포함되어 있지 않다. 따라서 ret == 0일 때, 즉
	 * 서명 검증이 실패한 경우 따로 에러 메시지를 출력한다. */
	if (!ret)
		ERR_print_errors(bio_err);
	if (!ret)
		fprintf(stderr, "signature verification failed!\n");
	if (kcdsa != NULL) KCDSA_free(kcdsa);
	if (kcdsa_sig != NULL) KCDSA_SIG_free(kcdsa_sig);
	CRYPTO_cleanup_all_ex_data();
	ERR_remove_state(0);
	ERR_free_strings();
	CRYPTO_mem_leaks(bio_err);
	if (bio_err != NULL)
		{
		BIO_free(bio_err);
		bio_err = NULL;
		}
	exit(!ret);
	return(0);
	}

static int kcdsa_cb(int p, int n, BN_GENCB *arg)
	{
	char c='*';
	static int ok=0;

	if (p == 0) { c='.'; };
	if (p == 1) c='+';
	if (p == 2) { c='*'; ok++; }
	if (p == 3) c='\n';
	BIO_write(arg->arg,&c,1);
	(void)BIO_flush(arg->arg);

	if (ok>1 && (p == 0))
		{
		BIO_printf((BIO *)arg->arg,"error in kcdsa_cb\n");
		return 0;
		}
	return 1;
	}
